package METRICS;

# ------------------------------------------------------------------------

#Copyright (C) 2005 Enrique Amig and Jess Gimnez

#This library is free software; you can redistribute it and/or
#modify it under the terms of the GNU Lesser General Public
#License as published by the Free Software Foundation; either
#version 2.1 of the License, or (at your option) any later version.

#This library is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#Lesser General Public License for more details.

#You should have received a copy of the GNU Lesser General Public
#License along with this library; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# ------------------------------------------------------------------------

use strict;
use Data::Dumper;
use IQ::COMMON;
use IQ::IQXML;
use IQ::QARLA;
use IQ::BLEU;
use IQ::GTM;
use IQ::NIST;
use IQ::METEOR;
use IQ::ROUGE;

$METRICS::rBLEU = $BLEU::rBLEU;
$METRICS::rGTM = $GTM::rGTM;
$METRICS::rMETEOR = $METEOR::rMETEOR;
$METRICS::rNIST = $NIST::rNIST;
$METRICS::rROUGE = $ROUGE::rROUGE;
$METRICS::rDPeng = { };
$METRICS::rDPspacat = { };
$METRICS::rSPeng = { };
$METRICS::rSPspacat = { };

sub criterion {
  my $A = substr($a, 0, 1); 
  my $B = substr($b, 0, 1);
  if ($A eq $B) { my @lA = split(/[RDS]/, $a); my @lB = split(/[RDS]/, $b); return ($lA[1] <=> $lB[1]); }
  else { return ($A cmp $B); } 
}

sub doMetrics {
   #description _ MLT evaluation: * generates SGML evaluation files
   #                              * computes GTM f1 (by calling Proteus java gtm) -> e = 1..3
   #                              * computes BLEU score (by calling NIST mteval script) -> n = 4
   #                              * computes NIST score (by calling NIST mteval script) -> n = 5
   #                              * computes METEOR score
   #                              * computes ROUGE
   #param1  _ source file 
   #param2  _ candidate file
   #param3  _ reference file
   #param4  _ remake reports? (1 - yes :: 0 - no)
   #param5  _ target NAME (base filename)
   #param6  _ reference NAME (base filename)
   #param7  _ TOOL directory
   #param8  _ set of metrics
   #param9  _ shallow parser (object)
   #param10 _ language (SPA | CAT | ENG)
   #param11 _ case (lc | all | ...)
   #param12 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $ref = shift;
   my $remakeREPORTS = shift;
   my $TGT = shift;
   my $REF = shift;
   my $tools = shift;
   my $M = shift;
   my $parser = shift;
   my $L = shift;
   my $C = shift;
   my $verbose = shift;

   if ($verbose > 1) { print STDERR "computing MT-scores [$TGT] vs [$REF]...\n"; }
   elsif ($verbose == 1) { print STDERR "$TGT-$REF ["; }

   if (!(-e "./$TGT")) { system "mkdir ./$TGT"; }
   if (!(-e "./$TGT/$REF")) { system "mkdir ./$TGT/$REF"; }

   BLEU::doBLEU($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, "", $verbose);
   GTM::doGTM($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, "", $verbose);
   METEOR::doMETEOR($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   NIST::doNIST($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, "", $verbose);
   ROUGE::doROUGE($src, $out, $ref, $remakeREPORTS, $TGT, $REF, $tools, $M, 1, "", $verbose); # + stemming

   if ($verbose == 1) { print STDERR "]\n"; }
}

sub doMultiMetrics {
   #description _ MLT evaluation: for multiple references
   #                              * generates SGML evaluation files
   #                              * computes GTM f1 (by calling Proteus java gtm) -> e = 1..3
   #                              * computes BLEU score (by calling NIST mteval script) -> n = 4
   #                              * computes NIST score (by calling NIST mteval script) -> n = 5
   #                              * computes METEOR score
   #                              * computes ROUGE
   #param1  _ source file 
   #param2  _ candidate file
   #param3  _ reference file(s) (hash ref)
   #param4  _ reference NAME(s) (list ref)
   #param5  _ remake reports? (1 - yes :: 0 - no)
   #param6  _ SYSTEM NAME (base filename)
   #param7  _ TOOL directory
   #param8  _ set of metrics
   #param9  _ shallow parser (object)
   #param10 _ language (SPA | CAT | ENG)
   #param11 _ case (lc | all | ...)
   #param12 _ verbosity (0/1)

   my $src = shift;
   my $out = shift;
   my $Href = shift;
   my $Lref = shift;
   my $remakeREPORTS = shift;
   my $TGT = shift;
   my $tools = shift;
   my $M = shift;
   my $parser = shift;
   my $L = shift;
   my $C = shift;
   my $verbose = shift;

   my %HREF;
   foreach my $r (sort criterion @{$Lref}) { if (exists($Href->{$r})) { $HREF{$r} = $Href->{$r}; } }
   my $REF = join("_", sort criterion keys %HREF);

   if (!(-e "./$TGT")) { system "mkdir ./$TGT"; }
   if (!(-e "./$TGT/$REF")) { system "mkdir ./$TGT/$REF"; }

   if ($verbose > 1) { print STDERR "computing MT-scores [$TGT]...\n"; }
   elsif ($verbose == 1) { print STDERR "$TGT-$REF ["; }

   BLEU::doMultiBLEU($src, $out, \%HREF, $remakeREPORTS, $TGT, $REF, $tools, $M, "", $verbose);
   GTM::doMultiGTM($src, $out, \%HREF, $remakeREPORTS, $TGT, $REF, $tools, $M, "", $verbose);
   METEOR::doMultiMETEOR($src, $out, \%HREF, $remakeREPORTS, $TGT, $REF, $tools, $M, $verbose);
   NIST::doMultiNIST($src, $out, \%HREF, $remakeREPORTS, $TGT, $REF, $tools, $M, "", $verbose);
   ROUGE::doMultiROUGE($src, $out, \%HREF, $remakeREPORTS, $TGT, $REF, $tools, $M, 1, "", $verbose); # + stemming

   if ($verbose == 1) { print STDERR "]\n"; }
}

# *************************************************************************************
# ************************* METRIC REPORTS ********************************************
# *************************************************************************************

sub read_metrics {
    #description _ read an xml IQREPORT file onto memory
    #param1  _ set of metrics
    #param2  _ set of systems
    #param3  _ set of references
    #param4  _ set of segments (topics)
    #param5  _ hash of scores (outside QARLA)
    #param6  _ level of granularity ('sys' for system; 'seg' for segment)
    #param7  _ verbosity (0/1)

    my $M = shift;
    my $S = shift;
    my $R = shift;
    my $T = shift;
    my $hOQ = shift;
    my $G = shift;
    my $verbose = shift;

    my @topics;
    foreach my $t (@{$T}) { $topics[$t] = 1; }
    
    my $NSEGMENTS = 0;
    my $REF = join("_", sort criterion @{$R});

    foreach my $s (sort criterion @{$S}) {
       if  ($verbose == 1) { print STDERR "..$s vs $REF ["; }
       foreach my $m (sort @{$M}) {
          if  ($verbose == 1) { print STDERR "..$m"; }
          $NSEGMENTS = IQXML::read_report($s, $REF, $m, "./$s/$REF/$m.$COMMON::XMLEXT", $hOQ, \@topics, $G, $verbose);
       }
       if  ($verbose == 1) { print STDERR "]\n"; }
    }

    return $NSEGMENTS;
}

sub read_write_metrics {
    #description _ reads all individual xml IQREPORT files and prints a table of similarities
    #              onto the given file
    #param1 _ filehandle
    #param2 _ target
    #param3 _ reference
    #param4 _ metrics
    #param5 _ verbosity (0/1)

    my $file = shift;
    my $T = shift;
    my $R = shift;
    my $M = shift;
    my $verbose = shift;

    foreach my $m (sort keys %{$M}) {
       IQXML::read_write_report($T, $R, $m, "./$T/$R/$m.$COMMON::XMLEXT", $file, $verbose);
    }
}

sub print_metrics {
    #description _ printing of metric scores inside/outside QARLA onto stdout
    #param1  _ hash of scores (outside QARLA)
    #param2  _ hash of scores (inside QARLA)
    #param3  _ system name
    #param4  _ level of granularity ('sys' for system; 'seg' for segment)
    #param5  _ metric set
    #param6  _ set of segments (topics)
    #param7  _ set of references (models)
    #param8  _ do metrics outside QARLA (0/1)
    #param9  _ do QUEEN (0/1)
    #param10 _ do QUEEN-TT (0/1)
    #param11 _ #SEGMENTS
    #param12 _ verbosity (0/1)

    my $hOQ = shift;
    my $hIQ = shift;
    my $system = shift;
    my $G = shift;
    my $M = shift;
    my $T = shift;
    my $R = shift;
    my $doOQ = shift;
    my $doQUEEN = shift;
    my $doQUEENtt = shift;
    my $nSEGMENTS = shift;
    my $verbose = shift;


    if ($doOQ or $doQUEEN or $doQUEENtt) {

       my @auxSYS = ($system);
       my $ref = join("_", sort criterion @{$R});

       if ($G eq 'sys') { # G == 'sys'
          my @scores;
          if ($doOQ) {
             foreach my $m (sort @{$M}) { push(@scores, sprintf("%10.4f", $hOQ->{$m}->{$system}->{$ref})); }
          }
          if ($doQUEEN) {
             my ($QUEEN, $lQ) = QARLA::QUEEN($hIQ, $M, $T, \@auxSYS, $R, $verbose);
             push(@scores, sprintf("%10.4f", $QUEEN));
          }
          if ($doQUEENtt) {
             my ($QUEENtt, $lQ) = QARLA::QUEEN_TT($hIQ, $M, $T, $T, \@auxSYS, $R, $verbose);
             push(@scores, sprintf("%10.4f", $QUEENtt));
          }
          printf "%-6s %s\n", $system, join(" ", @scores);
       }
       else { # G == 'seg'
          my @topics;
          foreach my $t (@{$T}) { $topics[$t] = 1; }
          my $QUEEN;
          my $QUEENtt;
          my $lQ;
          my $lQtt;
          if ($doQUEEN) { ($QUEEN, $lQ) = QARLA::QUEEN($hIQ, $M, $T, \@auxSYS, $R, $verbose); }
          if ($doQUEENtt) { ($QUEENtt, $lQtt) = QARLA::QUEEN_TT($hIQ, $M, $T, $T, \@auxSYS, $R, $verbose); }
          my $segment = 1;
	  if (($nSEGMENTS == 0) and $doQUEEN) { $nSEGMENTS = scalar(@{$lQ}); }
	  if (($nSEGMENTS == 0) and $doQUEENtt) { $nSEGMENTS = scalar(@{$lQtt}); }
          while ($segment <= $nSEGMENTS) {
             if ($topics[$segment]) {
                my @scores;
                if ($doOQ) {
                   foreach my $m (sort @{$M}) { push(@scores, sprintf("%10.4f", $hOQ->{$m}->[$segment]->{$system}->{$ref})); }
                }
                if ($doQUEEN) { push(@scores, sprintf("%10.4f", $lQ->[$segment])); }
                if ($doQUEENtt) { push(@scores, sprintf("%10.4f", $lQtt->[$segment])); }
                printf "%-14s %s\n", $system.":".$segment, join(" ", @scores);
	     }
             $segment++;
          }
       }
    }
}

sub print_rankings
{
    #description _ printing of ranking lists of system scores on a metric-by-metric basis
    #param1  _ metrics outside QARLA (hash ref)
    #param2  _ metrics inside QARLA (hash ref)
    #param3  _ level of granularity ('sys'/'seg')
    #param4  _ metric set
    #param5  _ set of segments (topics)
    #param6  _ set of systems
    #param7  _ set of references (models)
    #param8  _ do metrics outside QARLA (0/1)
    #param9  _ do QUEEN (0/1)
    #param10 _ do QUEEN-TT (0/1)
    #param11 _ #SEGMENTS
    #param12 _ include references as well
    #param13 _ verbosity (0/1)

    my $hOQ = shift;
    my $hIQ = shift;
    my $G = shift;
    my $M = shift;
    my $T = shift;
    my $S = shift;
    my $R = shift;
    my $doOQ = shift;
    my $doQUEEN = shift;
    my $doQUEENtt = shift;
    my $nSEGMENTS = shift;
    my $doREF = shift;
    my $verbose = shift;

    if ($doOQ or $doQUEEN or $doQUEENtt) {

       my $ref = join("_", sort criterion @{$R});

       if ($G eq 'sys') { # G == 'sys'
          if ($doOQ) {
             foreach my $m (sort @{$M}) {
                print "RANK=$m\n";
                foreach my $system (sort criterion @{$S}) {
                   printf("%-6s %10.4f\n", $system, $hOQ->{$m}->{$system}->{$ref});
	        }
                if ($doREF) {
                   foreach my $reference (sort criterion @{$R}) {
                      my @refs;
                      foreach my $r (sort criterion @{$R}) { if ($r ne $reference) { push(@refs, $r); } }
                      my $REF = join("_", sort criterion @refs);
                      printf("%-6s %10.4f\n", $reference, $hOQ->{$m}->{$reference}->{$REF});
		   }
	        }
   	     }
          }
          if ($doQUEEN) {
             print "RANK=QUEEN\n";
             foreach my $system (sort criterion @{$S}) {
                my @auxSYS = ($system);
                my ($QUEEN, $lQ) = QARLA::QUEEN($hIQ, $M, $T, \@auxSYS, $R, $verbose);
                printf("%-6s %10.4f\n", $system, $QUEEN);
             }
             if ($doREF) {
                foreach my $reference (sort criterion @{$R}) {
                   my @auxREF = ($reference);
                   my @refs;
                   foreach my $r (sort criterion @{$R}) { if ($r ne $reference) { push(@refs, $r); } }
                   my ($QUEEN, $lQ) = QARLA::QUEEN($hIQ, $M, $T, \@auxREF, \@refs, $verbose);
                   printf("%-6s %10.4f\n", $reference, $QUEEN);
	        }
             }
          }
          if ($doQUEENtt) {
             print "RANK=QUEEN-TT\n";
             foreach my $system (sort criterion @{$S}) {
                my @auxSYS = ($system);
                my ($QUEENtt, $lQ) = QARLA::QUEEN_TT($hIQ, $M, $T, $T, \@auxSYS, $R, $verbose);
                printf("%-6s %10.4f\n", $system, $QUEENtt);
             }
             if ($doREF) {
                foreach my $reference (sort criterion @{$R}) {
                   my @auxREF = ($reference);
                   my @refs;
                   foreach my $r (sort criterion @{$R}) { if ($r ne $reference) { push(@refs, $r); } }
                   my ($QUEENtt, $lQ) = QARLA::QUEEN_TT($hIQ, $M, $T, $T, \@auxREF, \@refs, $verbose);
                   printf("%-6s %10.4f\n", $reference, $QUEENtt);
	        }
             }
          }
       }
       else { # G == 'seg'
          my @topics;
          foreach my $t (@{$T}) { $topics[$t] = 1; }

          if ($doOQ) {
             foreach my $m (sort @{$M}) {
                print "RANK=$m\n";
                foreach my $system (sort criterion @{$S}) {
                   my $segment = 1;
                   while ($segment <= $nSEGMENTS) {
		      if ($topics[$segment]) {
                         printf("%-14s %10.4f\n", $system.":".$segment, $hOQ->{$m}->[$segment]->{$system}->{$ref});
		      }
                      $segment++;
                   }
	        }
                if ($doREF) {
                   foreach my $reference (sort criterion @{$R}) {
                      my @refs;
                      foreach my $r (sort criterion @{$R}) { if ($r ne $reference) { push(@refs, $r); } }
                      my $REF = join("_", sort criterion @refs);
                      my $segment = 1;
                      while ($segment <= $nSEGMENTS) {
		         if ($topics[$segment]) {
                            printf("%-14s %10.4f\n", $reference.":".$segment, $hOQ->{$m}->[$segment]->{$reference}->{$REF});
 	  	         }
                         $segment++;
		      }
                   }
	        }
  	     }
          }
          if ($doQUEEN) {
             print "RANK=QUEEN\n";
             foreach my $system (sort criterion @{$S}) {
                my @auxSYS = ($system);
                my ($QUEEN, $lQ) = QARLA::QUEEN($hIQ, $M, $T, \@auxSYS, $R, $verbose);
 	        if (($nSEGMENTS == 0) and $doQUEEN) { $nSEGMENTS = scalar(@{$lQ}); }
                my $segment = 1;
                while ($segment <= $nSEGMENTS) {
  	           if ($topics[$segment]) { printf("%-14s %10.4f\n", $system.":".$segment, $lQ->[$segment]); }
                   $segment++;
                }
	     }
             if ($doREF) {
                foreach my $reference (sort criterion @{$R}) {
                   my @refs;
                   my @auxREF = ($reference);
                   foreach my $r (sort criterion @{$R}) { if ($r ne $reference) { push(@refs, $r); } }
                   my ($QUEEN, $lQ) = QARLA::QUEEN($hIQ, $M, $T, \@auxREF, \@refs, $verbose);
   	           if (($nSEGMENTS == 0) and $doQUEEN) { $nSEGMENTS = scalar(@{$lQ}); }
                   my $segment = 1;
                   while ($segment <= $nSEGMENTS) {
 	              if ($topics[$segment]) { printf("%-14s %10.4f\n", $reference.":".$segment, $lQ->[$segment]); }
                      $segment++;
		   }
                }
	     }
          }
          if ($doQUEENtt) {
             print "RANK=QUEEN-TT\n";
             foreach my $system (sort criterion @{$S}) {
                my @auxSYS = ($system);
                my ($QUEEN, $lQtt) = QARLA::QUEEN_TT($hIQ, $M, $T, $T, \@auxSYS, $R, $verbose);
 	        if (($nSEGMENTS == 0) and $doQUEENtt) { $nSEGMENTS = scalar(@{$lQtt}); }
                my $segment = 1;
                while ($segment <= $nSEGMENTS) {
  	           if ($topics[$segment]) { printf("%-14s %10.4f\n", $system.":".$segment, $lQtt->[$segment]); }
                   $segment++;
                }
	     }
             if ($doREF) {
                foreach my $reference (sort criterion @{$R}) {
                   my @refs;
                   my @auxREF = ($reference);
                   foreach my $r (sort criterion @{$R}) { if ($r ne $reference) { push(@refs, $r); } }
                   my ($QUEENtt, $lQtt) = QARLA::QUEEN_TT($hIQ, $M, $T, $T, \@auxREF, \@refs, $verbose);
   	           if (($nSEGMENTS == 0) and $doQUEENtt) { $nSEGMENTS = scalar(@{$lQtt}); }
                   my $segment = 1;
                   while ($segment <= $nSEGMENTS) {
 	              if ($topics[$segment]) {
                         printf("%-14s %10.4f\n", $reference.":".$segment, $lQtt->[$segment]);
		      }
                      $segment++;
		   }
                }
	     }
          }
       }       
    }
}

sub print_avg_quality {
    #description _ printing of average metric scores (system level) outside QARLA onto stdout
    #param1  _ hash of scores (outside QARLA)
    #param2  _ system name
    #param3  _ metric set
    #param4  _ set of segments (topics)
    #param5  _ set of references (models)
    #param6  _ do metrics outside QARLA (0/1)
    #param7  _ verbosity (0/1)

    my $hOQ = shift;
    my $system = shift;
    my $M = shift;
    my $T = shift;
    my $R = shift;
    my $nSEGMENTS = shift;
    my $verbose = shift;

    my $ref = join("_", sort criterion @{$R});

    my %AVG;
    my $segment = 1;
    while ($segment <= $nSEGMENTS) {
       foreach my $m (sort @{$M}) {
	   $AVG{$m} += $hOQ->{$m}->[$segment]->{$system}->{$ref};
       }
       $segment++;
    }
    foreach my $m (sort keys %AVG) {
       printf "%-14s %-10s %10.4f\n", $system, $m, $AVG{$m} / $nSEGMENTS;
    }
}

sub add_avg_quality {
    #description _ printing of average metric scores (system level) outside QARLA onto stdout
    #param1  _ hash of scores (outside QARLA)
    #param2  _ system name
    #param3  _ metric set
    #param4  _ set of segments (topics)
    #param5  _ set of references (models)
    #param6  _ do metrics outside QARLA (0/1)
    #param7  _ average quality hash structure
    #param8  _ verbosity (0/1)

    my $hOQ = shift;
    my $system = shift;
    my $M = shift;
    my $T = shift;
    my $R = shift;
    my $nSEGMENTS = shift;
    my $hAVGQ = shift;
    my $verbose = shift;

    my $ref = join("_", sort criterion @{$R});

    my @topics;
    foreach my $t (@{$T}) { $topics[$t] = 1; }
#    print Dumper \@topics;

    #my %AVG;
    my $segment = 1;
    while ($segment <= $nSEGMENTS) {
       if ($topics[$segment]) {
          foreach my $m (sort @{$M}) {
             $hAVGQ->{$system}->{$m}->[0] += $hOQ->{$m}->[$segment]->{$system}->{$ref};
             $hAVGQ->{$system}->{$m}->[1]++;
          }
       }
       $segment++;
    }
}

sub compute_Pking_Porange {
    #description _ computes P_king and/or P_orange probabilities
    #param1  _ source
    #param2  _ hash of systems
    #param3  _ hash of references
    #param4  _ hash of metrics
    #param5  _ set of systems (peers)
    #param6  _ set of references (models)
    #param7  _ set of segments (topics)
    #param8  _ set of metrics
    #param9  _ IQMT tools directory pathname
    #param10 _ parser object
    #param11 _ LANGUAGE (SPA, CAT, ENG...)
    #param12 _ case (lc | all | ...)
    #param13 _ remake reports? (0/1)
    #param14 _ verbosity (0/1)

    my $src = shift;
    my $Hsystems = shift;
    my $Hrefs = shift;
    my $Hmetrics = shift;
    my $S = shift;
    my $R = shift;
    my $T = shift;
    my $M = shift;
    my $IQMT = shift;
    my $parser = shift;
    my $LANG = shift;
    my $CASE = shift;
    my $remake = shift;
    my $verbose = shift;

    if (scalar(@{$R}) < 2) { die "[IQMT] unsufficient number of reference translations!!! (at least 2 references required)\n"; }
    foreach my $RR (sort criterion @{$R}) {
       my @refs;
       foreach my $r (sort criterion @{$R}) { if ($r ne $RR) { push(@refs, $r); } }
       foreach my $t (sort criterion @{$S}) {
          METRICS::doMultiMetrics($src, $Hsystems->{$t}, $Hrefs, \@refs, $remake, $t, $IQMT, $Hmetrics, $parser, $LANG, $CASE, $verbose);
       }
       METRICS::doMultiMetrics($src, $Hrefs->{$RR}, $Hrefs, \@refs, $remake, $RR, $IQMT, $Hmetrics, $parser, $LANG, $CASE, $verbose);
    }

    # -------------------------------------------------

    my @topics;
    foreach my $t (@{$T}) { $topics[$t] = 1; }

    my %COUNT1;
    my %TOTAL1;
    my %COUNT2;
    my %TOTAL2;
    foreach my $RR (sort criterion @{$R}) {
       my @refs;
       foreach my $r (@{$R}) { if ($r ne $RR) { push(@refs, $r); } }
       my $ref = join("_", sort criterion @refs);
       foreach my $m (sort @{$M}) {

          my %OQ;
          my @auxMETRIC = ($m);
          foreach my $t (sort criterion @{$S}) {
             my @auxSYS = ($t);
             my $NSEGMENTS = METRICS::read_metrics(\@auxMETRIC, \@auxSYS, \@refs, $T, \%OQ, 'seg', $verbose);
          }
          my @auxREF = ($RR);
          my $NSEGMENTS = METRICS::read_metrics(\@auxMETRIC, \@auxREF, \@refs, $T, \%OQ, 'seg', $verbose);
          my $segment = 1;
          my $nSEGMENTS = scalar(@{$OQ{$m}}) - 1;
          while ($segment <= $nSEGMENTS) { 
             if ($topics[$segment]) {
	        my $Rquality =  $OQ{$m}->[$segment]->{$RR}->{$ref};
                my $hit = 1;
                my $max = 0;
                if ($verbose) { printf "%-14s %-14s %-10d %10.4f", $RR, $m, $segment, $Rquality; }
                foreach my $s (sort criterion @{$S}) {
                   my $Squality =  $OQ{$m}->[$segment]->{$s}->{$ref};
                   if ($verbose) { printf "%10.4f", $Squality; }
                   if ($Rquality <= $Squality) { $hit = 0; } #### 1 P_king
                   $TOTAL2{$m} ++; #### 2
                   if ($Rquality > $Squality) { $COUNT2{$m}++; } #### 2 P_orange
                }
                if ($hit) { $COUNT1{$m}++; } #### 1 P_king
                if ($verbose) { print "\n"; }
	     }
             $segment++;
          }
          $TOTAL1{$m} += scalar(@{$T}); #### 1
       }
    }

    my %Pking;
    my %Porange;
    foreach my $m (sort @{$M}) {
       $Porange{$m} = $COUNT2{$m} / $TOTAL2{$m};
       $Pking{$m} = $COUNT1{$m} / $TOTAL1{$m};
    }

    return(\%Pking, \%Porange);
}

1;
