# License Start: 
#                    Carnegie Mellon University                      
#                      Copyright (c) 2004                            
#                       All Rights Reserved.                         
#
# Permission is hereby granted, free of charge, to use and distribute
# this software and its documentation without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of this work, and to    
# permit persons to whom this work is furnished to do so, subject to 
# the following conditions:                                          
#  1. The code must retain the above copyright notice, this list of  
#     conditions and the following disclaimer.                       
#  2. Any modifications must be clearly marked as such.              
#  3. Original authors' names are not deleted.                       
#  4. The authors' names are not used to endorse or promote products 
#     derived from this software without specific prior written      
#     permission.                                                    
#
# CARNEGIE MELLON UNIVERSITY AND THE CONTRIBUTORS TO THIS WORK       
# DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING    
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT 
# SHALL CARNEGIE MELLON UNIVERSITY NOR THE CONTRIBUTORS BE LIABLE    
# FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES  
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN 
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,        
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF     
# THIS SOFTWARE.                                                     
#
# Author: Satanjeev "Bano" Banerjee satanjeev@cmu.edu
# Author: Alon Lavie alavie@cs.cmu.edu
#
# License End.



package wn_synonymy;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(setUpDataStructures);

my $wn; # will contain the wordnet object

# this will create the alignment-ready data structures by matching
# words that are synonyms of each other
sub setUpDataStructures {
  my $firstStringWordsRef = shift;
  my $secondStringWordsRef = shift;
  my $string2OriginalPosRef = shift;
  my $string2MatchedPosRef = shift;
  my $multiChoiceWordIndexesRef = shift;
  my $posChoicesRef = shift;
  my $alreadyAlignedFirstStringRef = shift;
  my $alreadyAlignedSecondStringRef = shift;
  $wn = shift;

  my $i;

  my %firstStringWordsUsedUp = ();
  my $index = 0;
    
  for ($i = 0; $i <= $#{$secondStringWordsRef}; $i++) {
    next if (defined ${$alreadyAlignedSecondStringRef}{$i});
    for ($j = 0; $j <= $#{$firstStringWordsRef}; $j++) {
      next if (defined ${$alreadyAlignedFirstStringRef}{$j});
      next if (defined $firstStringWordsUsedUp{$j});
      next unless (wn_synonyms($$secondStringWordsRef[$i], $$firstStringWordsRef[$j]));
	
      # ah so they are synonyms
      $$string2OriginalPosRef[$index] = $i;
      $$string2MatchedPosRef[$index] = $j;
      $index++; 	        
	
      $firstStringWordsUsedUp{$j} = 1;
      last;
    }
  }
}

sub wn_synonyms {
  my $w1 = shift;
  my $w2 = shift;
  
  my %offsets1 = ();
  my %offsets2 = ();
  
  # get all the possible forms 
  my @forms = $wn->validForms($w1);
  my $form = "";
  foreach $form (@forms) {
    my @synsets = $wn->querySense($form, "syns");
    my $synset = "";
    foreach $synset(@synsets) {
      my $offset = $wn->offset($synset);
      $offsets1{$offset} = 1;
    }
  }

  @forms = $wn->validForms($w2);
  $form = "";
  foreach $form (@forms) {
    my @synsets = $wn->querySense($form, "syns");
    my $synset = "";
    foreach $synset(@synsets) {
      my $offset = $wn->offset($synset);
      $offsets2{$offset} = 1;
    }
  }

  # check if there is any overlap
  foreach (keys %offsets1) { 
    return (1) if (defined $offsets2{$_}); 
  }
  
  return(0);
}

1;
